#-*- coding: UTF-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Noarch
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

import os
from Spec import *

class Noarch(Spec):
    """
    Noarch Class
        Write the spec file for noarch package.
    """
    
    def __init__(self, package):
        self.package = package
    
    def writeSpec(self, specname):
        ''' Write the spec file '''

        print "* Spec generated: \n  %s/%s" %(os.getcwd(), specname)
        specfile = open(specname, 'w')
        spec = ""
        if self.package.packager.bioc:
            spec += "%global BioC      "+"%s\n" %(self.package.packager.bioc)
        spec += """%global packname  """ + self.package.name + """

Name:             R-%{packname}
Version:          """ + self.package.description.version + """
Release:          1%{?dist}
Summary:          """ + self.package.description.summary + """

Group:            Applications/Engineering 
License:          """ + self.package.description.license + """ 
URL:              """ + self.package.URL + """
Source0:          """ + self.package.source0 + """
BuildRoot:        %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)

BuildArch:        noarch
"""
        if self.package.description.depends != "" and self.package.description.suggests !="":
            spec += """
Requires:         """ + self.package.description.depends + " " + self.package.description.suggests
        spec+="""
Requires(post):   R
Requires(post):   R
Requires(postun): R
BuildRequires:    R-devel tetex-latex """ + self.package.description.depends + " " + self.package.description.suggests + """

%description
""" + self.package.description.description + """

%prep
%setup -q -c -n %{packname}

%build

%install
rm -rf %{buildroot}


# x86/x86_64 -> Architecture dependent package
#mkdir -p %{buildroot}%{_libdir}/R/library 
#R CMD INSTALL %{packname} -l %{buildroot}%{_libdir}/R/library
## Clean up in advance of check
#test -d %{packname}/src && (cd %{packname}/src; rm -f *.o *.so)
#rm -rf %{buildroot}%{_libdir}/R/library/R.css

# noarch -> Architecture independent package
mkdir -p %{buildroot}%{_datadir}/R/library 
R CMD INSTALL %{packname} -l %{buildroot}%{_datadir}/R/library
# Clean up in advance of check
test -d %{packname}/src && (cd %{packname}/src; rm -f *.o *.so)
rm -rf %{buildroot}%{_datadir}/R/library/R.css

%check
%{_bindir}/R CMD check %{packname}

%clean
rm -rf %{buildroot}

%post
%{_R_make_search_index}

%postun
%{_R_make_search_index}

%files
%defattr(-, root, root, -)
#i386 arch
#%dir %{_libdir}/R/library/%{packname}
#%doc %{_libdir}/R/library/%{packname}/latex
#%doc %{_libdir}/R/library/%{packname}/doc
#%doc %{_libdir}/R/library/%{packname}/html
#%doc %{_libdir}/R/library/%{packname}/DESCRIPTION
#%doc %{_libdir}/R/library/%{packname}/man
#%doc %{_libdir}/R/library/%{packname}/NEWS

#noarch
%dir %{_datadir}/R/library/%{packname}
%doc %{_datadir}/R/library/%{packname}/latex
%doc %{_datadir}/R/library/%{packname}/doc
%doc %{_datadir}/R/library/%{packname}/html
%doc %{_datadir}/R/library/%{packname}/DESCRIPTION
%doc %{_datadir}/R/library/%{packname}/man
%doc %{_datadir}/R/library/%{packname}/NEWS
%{_datadir}/R/library/%{packname}/CONTENTS
%{_datadir}/R/library/%{packname}/INDEX
%{_datadir}/R/library/%{packname}/NAMESPACE
%{_datadir}/R/library/%{packname}/Meta
%{_datadir}/R/library/%{packname}/R
%{_datadir}/R/library/%{packname}/R-ex
%{_datadir}/R/library/%{packname}/help

%changelog
* """ + self.package.date + " " + self.package.packager.name + " <" + self.package.packager.email + "> " + self.package.description.version + """-1
- initial package for Fedora
"""
        specfile.write(spec)
        specfile.close()
        print "* Done"
    
    def writeOut(self):
        print "*"*50
        print """TODO:
 * Check Summary
 * Check description
 * Check license
 * Add/Check Source0 and URL
 * Check Arch - We detected it as Noarch package but it needs to be checked, use rpmlint for it
 * Change Requires and BuildRequires
 * Change %files
 """
